@@ -16,7 +16,6 @@
  */
 
 #include "sysincludes.h"
-#include "msdos.h"
 #include "stream.h"
 #include "mtools.h"
 #include "fsP.h"
@@ -268,8 +267,8 @@ void printFatWithOffset(Stream_t *Stream, off_t offset) {
 	printf("%lu", (unsigned long) n);
 }
 
-static int normal_map(File_t *This, uint32_t where, uint32_t *len, int mode,
-		      mt_off_t *res)
+static int normal_map(File_t *This, uint32_t where, uint32_t *len,
+		      int isReadonly, mt_off_t *res)
 {
 	unsigned int offset;
 	size_t end;
@@ -285,13 +284,13 @@ static int normal_map(File_t *This, uint32_t where, uint32_t *len, int mode,
 	clus_size = Fs->cluster_size * Fs->sector_size;
 	offset = where % clus_size;
 
-	if (mode == MT_READ)
+	if (isReadonly)
 		maximize(*len, This->FileSize - where);
 	if (*len == 0 )
 		return 0;
 
 	if (This->FirstAbsCluNr < 2){
-		if( mode == MT_READ || *len == 0){
+		if( isReadonly || *len == 0){
 			*len = 0;
 			return 0;
 		}
@@ -333,7 +332,7 @@ static int normal_map(File_t *This, uint32_t where, uint32_t *len, int mode,
 		}
 		if(CurCluNr == RelCluNr + NrClu)
 			break;
-		if (NewCluNr > Fs->last_fat && mode == MT_WRITE){
+		if (NewCluNr > Fs->last_fat && !isReadonly){
 			/* if at end, and writing, extend it */
 			NewCluNr = get_next_free_cluster(_getFs(This), AbsCluNr);
 			if (NewCluNr == 1 ){ /* no more space */
@@ -362,7 +361,7 @@ static int normal_map(File_t *This, uint32_t where, uint32_t *len, int mode,
 
 	end = where + *len;
 	if(batchmode &&
-	   mode == MT_WRITE &&
+	   !isReadonly &&
 	   end >= This->FileSize) {
 		/* In batch mode, when writing at end of file, "pad"
 		 * to nearest cluster boundary so that we don't have
@@ -388,7 +387,7 @@ static int root_map(File_t *This, uint32_t where, uint32_t *len,
 {
 	Fs_t *Fs = _getFs(This);
 
-	if(Fs->dir_len * Fs->sector_size < where) {
+	if((Fs->dir_len + 0u) * Fs->sector_size < where) {
 		*len = 0;
 		errno = ENOSPC;
 		return -2;
@@ -413,7 +412,7 @@ static ssize_t read_file(Stream_t *Stream, char *buf, size_t ilen)
 	
 	Stream_t *Disk = _getFs(This)->head.Next;
 
-	err = This->map(This, This->where, &len, MT_READ, &pos);
+	err = This->map(This, This->where, &len, 1, &pos);
 	if(err <= 0)
 		return err;
 	ret = PREADS(Disk, buf, pos, len);
@@ -440,7 +439,7 @@ static ssize_t write_file(Stream_t *Stream, char *buf, size_t ilen)
 	} else
 		len = (uint32_t) ilen;
 	requestedLen = len;
-	err = This->map(This, This->where, &len, MT_WRITE, &pos);
+	err = This->map(This, This->where, &len, 0, &pos);
 	if( err <= 0)
 		return err;
 	if(batchmode)
@@ -485,7 +484,7 @@ static ssize_t pwrite_file(Stream_t *Stream, char *buf, mt_off_t where,
 
 static int month[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334,
 					  0, 0, 0 };
-static __inline__ time_t conv_stamp(struct directory *dir)
+static inline time_t conv_stamp(struct directory *dir)
 {
 	struct tm *tmbuf;
 	long tzone, dst;
@@ -673,6 +672,7 @@ static Stream_t *_internalFileOpen(Stream_t *Dir, unsigned int first,
 	This->head.refs++;
 
 	if(first != 1){
+		void *Result;
 		/* we use the illegal cluster 1 to mark newly created files.
 		 * do not manage those by hashtable */
 		init_head(&Pattern.head, &FileClass, &This->head);
@@ -683,8 +683,9 @@ static Stream_t *_internalFileOpen(Stream_t *Dir, unsigned int first,
 		Pattern.FirstAbsCluNr = first;
 		Pattern.loopDetectRel = 0;
 		Pattern.loopDetectAbs = first;
-		if(!hash_lookup(filehash, (T_HashTableEl) &Pattern,
-				(T_HashTableEl **)&File, 0)){
+		if(!hash_lookup(filehash, &Pattern,
+				&Result, 0)){
+			File=Result;
 			File->head.refs++;
 			This->head.refs--;
 			return (Stream_t *) File;
@@ -720,7 +721,7 @@ static Stream_t *_internalFileOpen(Stream_t *Dir, unsigned int first,
 
 	File->PreviousRelCluNr = 0xffff;
 	File->FileSize = size;
-	hash_add(filehash, (void *) File, &File->hint);
+	hash_add(filehash, File, &File->hint);
 	return (Stream_t *) File;
 }
 
@@ -739,7 +740,7 @@ static void bufferize(Stream_t **Dir)
 		return;
 	}
 	
-	BDir = buf_init(*Dir, 64*16384, 512, MDIR_SIZE);
+	BDir = buf_init(*Dir, 16384, 512, MDIR_SIZE);
 	if(!BDir){
 		FREE(Dir);
 		*Dir = NULL;
