@@ -21,16 +21,46 @@
  */
 
 #include "sysincludes.h"
-#include "msdos.h"
 #include "mtools.h"
 #include "vfat.h"
 #include "file.h"
 #include "dirCache.h"
 #include "dirCacheP.h"
 #include "file_name.h"
+#include "stream.h"
 
 /* #define DEBUG */
 
+
+#define VSE1SIZE 5
+#define VSE2SIZE 6
+#define VSE3SIZE 2
+
+
+struct vfat_subentry {
+	unsigned char id;		/* 0x40 = last; & 0x1f = VSE ID */
+	unsigned char text1[VSE1SIZE*2];
+	unsigned char attribute;	/* 0x0f for VFAT */
+	unsigned char hash1;		/* Always 0? */
+	unsigned char sum;		/* Checksum of short name */
+	unsigned char text2[VSE2SIZE*2];
+	unsigned char sector_l;		/* 0 for VFAT */
+	unsigned char sector_u;		/* 0 for VFAT */
+	unsigned char text3[VSE3SIZE*2];
+};
+
+#define VSE_LAST 0x40
+#define VSE_MASK 0x1f
+
+struct vfat_state {
+	wchar_t name[VBUFSIZE];
+	int status; /* is now a bit map of 32 bits */
+	int subentries;
+	unsigned char sum; /* no need to remember the sum for each entry,
+			    * it is the same anyways */
+	int present;
+};
+
 const char *short_illegals=";+=[]',\"*\\<>/?:|";
 const char *long_illegals = "\"*\\<>/?:|\005";
 
@@ -43,7 +73,7 @@ static void autorename(char *name,
 	unsigned int seqnum=0, maxseq=0;
 	char tmp;
 	char *p;
-	
+
 #ifdef DEBUG
 	printf("In autorename for name=%s.\n", name);
 #endif
@@ -117,21 +147,34 @@ void autorename_long(char *name, int bump)
 	autorename(name, '-', '\0', long_illegals, 255, bump);
 }
 
-
-static __inline__ int unicode_read(struct unicode_char *in,
-				   wchar_t *out, int num)
+/* If null encountered, set *end to 0x40 and write nulls rest of way
+ * 950820: Win95 does not like this!  It complains about bad characters.
+ * So, instead: If null encountered, set *end to 0x40, write the null, and
+ * write 0xff the rest of the way (that is what Win95 seems to do; hopefully
+ * that will make it happy)
+ */
+/* Always return num */
+static int unicode_write(wchar_t *in, unsigned char *out, int num, int *end_p)
 {
-	wchar_t *end_out = out+num;
+	int j;
 
-	while(out < end_out) {
-#ifdef HAVE_WCHAR_H
-		*out = in->lchar | ((in->uchar) << 8);
-#else
-		if (in->uchar)
-			*out = '_';
-		else
-			*out = in->lchar;
-#endif
+	for (j=0; j<num; ++j) {
+		if (*end_p)
+			/* Fill with 0xff */
+			out[0] = out[1] = 0xff;
+		else {
+			/* TODO / FIXME : handle case where wchat has more
+			 * than 2 bytes (i.e. bytes 2 or 3 are set.
+			 * ==> generate surrogate pairs?
+			 */
+			out[1] = (*in & 0xffff) >> 8;
+			out[0] = *in & 0xff;
+			if (! *in) {
+				*end_p = VSE_LAST;
+			}
+		}
+
+		++out;
 		++out;
 		++in;
 	}
@@ -139,7 +182,29 @@ static __inline__ int unicode_read(struct unicode_char *in,
 }
 
 
-void clear_vfat(struct vfat_state *v)
+static inline int unicode_read(unsigned char *in,
+				   wchar_t *out, int num)
+{
+	wchar_t *end_out = out+num;
+
+	while(out < end_out) {
+#ifdef HAVE_WCHAR_H
+		*out = in[0] | ((in[1]) << 8);
+#else
+		if (in[1])
+			*out = '_';
+		else
+			*out = (char) in[0];
+#endif
+		++out;
+		++in;
+		++in;
+	}
+	return num;
+}
+
+
+static void clear_vfat(struct vfat_state *v)
 {
 	v->subentries = 0;
 	v->status = 0;
@@ -155,7 +220,7 @@ void clear_vfat(struct vfat_state *v)
  * and adding in each character, from left to right, padding both
  * the name and extension to maximum length with spaces and skipping
  * the "." (hence always summing exactly 11 characters).
- * 
+ *
  * This exact algorithm is required in order to remain compatible
  * with Microsoft Windows-95 and Microsoft Windows NT 3.5.
  * Thanks to Jeffrey Richter of Microsoft Systems Journal for
@@ -163,7 +228,7 @@ void clear_vfat(struct vfat_state *v)
  *
  * David C. Niemi (niemi@tuxers.net) 95.01.19
  */
-static __inline__ unsigned char sum_shortname(const dos_name_t *dn)
+static inline unsigned char sum_shortname(const dos_name_t *dn)
 {
 	unsigned char sum;
 	const char *name=dn->base;
@@ -181,9 +246,9 @@ static __inline__ unsigned char sum_shortname(const dos_name_t *dn)
  * Return 1 if the VSEs comprise a valid long file name,
  * 0 if not.
  */
-static __inline__ void check_vfat(struct vfat_state *v, struct directory *dir)
+static inline void check_vfat(struct vfat_state *v, struct directory *dir)
 {
-	dos_name_t dn;;
+	dos_name_t dn;
 
 	if (! v->subentries) {
 #ifdef DEBUG
@@ -197,7 +262,7 @@ static __inline__ void check_vfat(struct vfat_state *v, struct directory *dir)
 
 	if (v->sum != sum_shortname(&dn))
 		return;
-	
+
 	if( (v->status & ((1<<v->subentries) - 1)) != (1<<v->subentries) - 1)
 		return; /* missing entries */
 
@@ -206,8 +271,11 @@ static __inline__ void check_vfat(struct vfat_state *v, struct directory *dir)
 	v->present = 1;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wsign-conversion"
+#ifdef HAVE_PRAGMA_DIAGNOSTIC
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wsign-conversion"
+#endif
+
 /* We have indeed different types for the entry slot
  * - the higher levels have a "signed" type, in order to accomodate
  *   reserved values for "root directory" entry, "not found" entries, and
@@ -284,9 +352,9 @@ int write_vfat(Stream_t *Dir, dos_name_t *shortname, char *longname,
 		num_vses = (uint8_t)((wlen + VSE_NAMELEN - 1)/VSE_NAMELEN);
 		for (vse_id = num_vses; vse_id; --vse_id) {
 			int end = 0;
-			
+
 			c = wlongname + (vse_id - 1) * VSE_NAMELEN;
-			
+
 			c += unicode_write(c, vse->text1, VSE1SIZE, &end);
 			c += unicode_write(c, vse->text2, VSE2SIZE, &end);
 			c += unicode_write(c, vse->text3, VSE3SIZE, &end);
@@ -297,7 +365,7 @@ int write_vfat(Stream_t *Dir, dos_name_t *shortname, char *longname,
 			       longname, vse_id, longname + (vse_id-1) * VSE_NAMELEN,
 			       start + num_vses - vse_id, start + num_vses);
 #endif
-			
+
 			entry.entry = start + num_vses - vse_id;
 			low_level_dir_write(&entry);
 		}
@@ -348,15 +416,15 @@ void dir_write(direntry_t *entry)
  * The following function translates a series of vfat_subentries into
  * data suitable for a dircache entry
  */
-static __inline__ void parse_vses(direntry_t *entry,			
+static inline void parse_vses(direntry_t *entry,
 				  struct vfat_state *v)
 {
 	struct vfat_subentry *vse;
 	unsigned char id, last_flag;
 	wchar_t *c;
-	
+
 	vse = (struct vfat_subentry *) &entry->dir;
-	
+
 	id = vse->id & VSE_MASK;
 	last_flag = (vse->id & VSE_LAST);
 	if (id > MAX_VFAT_SUBENTRIES) {
@@ -364,7 +432,7 @@ static __inline__ void parse_vses(direntry_t *entry,
 			id, entry->entry);
 		return;
 	}
-	
+
 /* 950819: This code enforced finding the VSEs in order.  Well, Win95
  * likes to write them in *reverse* order for some bizarre reason!  So
  * we pretty much have to tolerate them coming in any possible order.
@@ -384,17 +452,17 @@ static __inline__ void parse_vses(direntry_t *entry,
 		clear_vfat(v);
 		v->sum = vse->sum;
 	}
-	
+
 #ifdef DEBUG
 	if(v->status & (1 << (id-1)))
 		fprintf(stderr,
 			"parse_vses: duplicate VSE %d\n", vse->id);
 #endif
-	
+
 	v->status |= 1 << (id-1);
 	if(last_flag)
 		v->subentries = id;
-	
+
 #ifdef DEBUG
 	if (id > v->subentries)
 		/* simple test to detect entries preceding
@@ -411,7 +479,7 @@ static __inline__ void parse_vses(direntry_t *entry,
 #ifdef DEBUG
 	printf("Read VSE %d at %d, subentries=%d, = (%13ls).\n",
 	       id,entry->entry,v->subentries,&(v->name[VSE_NAMELEN * (id-1)]));
-#endif		
+#endif
 	if (last_flag)
 		*c = '\0';	/* Null terminate long name */
 }
@@ -447,7 +515,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 					endmarkSeen);
 			return addEndEntry(cache, direntry->entry);
 		}
-		
+
 		if (endmarkSeen || direntry->dir.name[0] == ENDMARK){
 				/* the end of the directory */
 			if(lookForFreeSpace) {
@@ -463,11 +531,11 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 			/* the main entry */
 			break;
 	}
-	
+
 	/* If we get here, it's a short name FAT entry, maybe erased.
 	 * thus we should make sure that the vfat structure will be
 	 * cleared before the next loop run */
-	
+
 	/* deleted file */
 	if (direntry->dir.name[0] == DELMARK) {
 		return addFreeEntry(cache, initpos,
@@ -510,7 +578,7 @@ static dirCacheEntry_t *vfat_lookup_loop_common(doscp_t *cp,
 			    newfile, &direntry->dir);
 }
 
-static __inline__ dirCacheEntry_t *vfat_lookup_loop_for_read(doscp_t *cp,
+static inline dirCacheEntry_t *vfat_lookup_loop_for_read(doscp_t *cp,
 							     direntry_t *direntry,
 							     dirCache_t *cache,
 							     int *io_error)
@@ -682,10 +750,10 @@ int vfat_lookup(direntry_t *direntry, const char *filename,
 	}
 }
 
-static __inline__ dirCacheEntry_t *vfat_lookup_loop_for_insert(doscp_t *cp,
-							       direntry_t *direntry,
-							       unsigned int initpos,
-							       dirCache_t *cache)
+static inline dirCacheEntry_t *vfat_lookup_loop_for_insert(doscp_t *cp,
+							   direntry_t *direntry,
+							   unsigned int initpos,
+							   dirCache_t *cache)
 {
 	dirCacheEntry_t *dce;
 	int io_error;
@@ -843,13 +911,15 @@ int lookupForInsert(Stream_t *Dir,
 		return 6;	/* Success */
 
 	/* Need more room.  Can we grow the directory? */
-	if(!isRootDir(Dir))		
+	if(!isRootDir(Dir))
 		return 5;	/* OK, try to grow the directory */
 
 	fprintf(stderr, "No directory slots\n");
 	return -1;
 }
-#pragma GCC diagnostic pop
+#ifdef HAVE_PRAGMA_DIAGNOSTIC
+# pragma GCC diagnostic pop
+#endif
 
 
 /* End vfat.c */
